﻿using System.Collections.Generic;
using System.IO;
using Syncfusion.DocIO;
using Syncfusion.DocIO.DLS;
using SyncfusionTest.Logic.Extensions;

namespace SyncfusionTest.Logic
{
    public class WordHandler
    {
        public Stream ReplaceText(Stream wordStream, IDictionary<string, string> textPairs)
        {
            using (var loadedDocument = new WordDocument(wordStream, FormatType.Automatic))
            {
                ReplaceTextInDocument(loadedDocument, textPairs);

                return loadedDocument.SaveAsMemoryStream();
            }
        }

        private void ReplaceTextInDocument(WordDocument loadedDocument, IDictionary<string, string> textPairs)
        {
            foreach (var keyValuePair in textPairs)
            {
                var selections = loadedDocument.FindAll(keyValuePair.Key, true, true);
                foreach (var selection in selections)
                {
                    var range = selection.GetAsOneRange();
                    range.Text = keyValuePair.Value;
                }
            }
        }

        public Stream MultiPageReplaceText(Stream wordStream, IEnumerable<IDictionary<string, string>> textPairList)
        {
            using (var loadedDocument = new WordDocument(wordStream, FormatType.Automatic))
            {
                var initial = true;

                foreach (var textPairs in textPairList)
                {
                    if (initial)
                    {
                        ReplaceTextInDocument(loadedDocument, textPairs);
                        initial = false;
                    }
                    else
                    {
                        wordStream.Position = 0;
                        using (var nestedDocument = new WordDocument(wordStream, FormatType.Automatic))
                        {
                            ReplaceTextInDocument(nestedDocument, textPairs);

                            loadedDocument.ImportContent(nestedDocument, ImportOptions.UseDestinationStyles);
                        }
                    }
                }

                return loadedDocument.SaveAsMemoryStream();
            }
        }

        public Stream FillTable(Stream wordStream, string[][] tableRowData, bool hasHeaderRow = true)
        {
            using (var loadedDocument = new WordDocument(wordStream, FormatType.Automatic))
            {
                IWTable table = null;

                // find the first table in the document
                foreach (WSection section in loadedDocument.Sections)
                {
                    if (section.Tables.Count > 0)
                    {
                        table = section.Tables[0];
                        break;
                    }
                }

                if (table != null)
                {
                    // start index from, make sure headers aren't overwritten by data
                    var rowStartIndex = hasHeaderRow ? 1 : 0;
                    for (var rowIndex = 0; rowIndex < tableRowData.Length; rowIndex++)
                    {
                        var rowData = tableRowData[rowIndex];

                        // check if file contains empty rows we can use
                        var row = table.Rows.Count - 1 < rowStartIndex + rowIndex
                            ? table.AddRow()
                            : table.Rows[rowStartIndex + rowIndex];

                        for (var cellIndex = 0; cellIndex < rowData.Length; cellIndex++)
                        {
                            var cellText = rowData[cellIndex];
                            var cell = row.Cells[cellIndex];

                            // check if cell already has (empty) content and re-use it
                            // to avoid double spacing
                            var paragraph = cell.Paragraphs.Count > 0
                                ? cell.Paragraphs[0]
                                : cell.AddParagraph();

                            paragraph.Text = cellText;
                        }
                    }
                }

                return loadedDocument.SaveAsMemoryStream();
            }
        }
    }
}
